/*******************************************************************************
 * ON Semiconductor                                                            *
 * NCV7383 Evaluation KIT                                                      *
 *                                                                             *
 * FlexRay_BD.c                                                                *
 * FlexRay Bus Driver handling functions                                       *
 *                                                                             *
 * Created by: Filip Brtan                                                     *
 * Date: 30.07.2012                                                            *
 *******************************************************************************/



/** S12X derivative information */ 
#include "mc9s12xf512.h"       

/** Macro definitions for general purpose I/O handling  */
#include "GPIO_macros.h"

/** Variable types and common definitions */
#include "typedefs.h" 

/** UNIFIED driver implementation */         
#include "Fr_UNIFIED.h" 

/** FlexRay Bus Driver functions definitions */
#include "FlexRay_BD.h"
				

Fr_BD_error_mode_type fr_bd_error_mode_A;
Fr_BD_error_mode_type fr_bd_error_mode_B;				
//				
void wait_100ns(UINT16 count100ns) {
  PITLD0 = count100ns;
  PITTF = 1;
  PITCFLMT_PFLMT0 = 1;
  PITFLT_PITFLT0 = 1;
  
  while(!PITTF_PITTF0);
}

// wait 1us
void wait_1us(UINT16 count1us) {
  PITLD0 = (word)((word)10 * count1us);
  PITTF = 1;
  PITCFLMT_PFLMT0 = 1;
  PITFLT_PITFLT0 = 1;
  
  while(!PITTF_PITTF0);
}

// wait 1ms
void wait_1ms(void) {
  PITLD0 = 9999;
  PITTF = 1;
  PITCFLMT_PFLMT0 = 1;
  PITFLT_PITFLT0 = 1;
  
  while(!PITTF_PITTF0);
}

/* Status Register readout */
Fr_bd_status_register fr_bd_read_status(Fr_channel_type channel) {
  Fr_bd_status_register Status;
  switch (channel) {
    case FR_CHANNEL_A : Status = fr_bd_read_status_A(); break;
    case FR_CHANNEL_B : Status = fr_bd_read_status_B();break;
    default: 
      Status.Flags.Word = 0xFF;
      Status.Version.Word = 0xFF;
      break;     
  }
  return Status; 
}

/* Channel A Status Register readout */
Fr_bd_status_register fr_bd_read_status_A(void) {
  Fr_bd_status_register Status;
  
  if (SPI1CR1_SPE){
    FR_CSN_A = 0;
    NOP();NOP();NOP();NOP();
    NOP();NOP();NOP();NOP();
    while (!SPI1SR_SPTEF); 
    SPI1DR = 0x0000;  // Start Transmittion
    
    while (!SPI1SR_SPIF);  /* Wait receive buffer empty? */
    Status.Flags.Word = SPI1DR;  /* Read data from receiver */

    NOP();NOP();NOP();NOP();
    NOP();NOP();NOP();NOP();
    FR_CSN_A = 1;
    Status.Version.Word = 0xFFFF; 
  } else {
    Status.Flags.Word = 0xFFFF;
    Status.Version.Word = 0xFFFF; 
  }
 
  return Status; 
}

/* Channel B Status Register readout */
Fr_bd_status_register fr_bd_read_status_B(void) {
   Fr_bd_status_register Status;
  
  if (SPI1CR1_SPE){
    FR_CSN_B = 0;
    NOP();NOP();NOP();NOP();
    NOP();NOP();NOP();NOP();
    while (!SPI1SR_SPTEF); 
    SPI1DR = 0x0000;  // Start Transmittion
    
    while (!SPI1SR_SPIF);  /* Wait receive buffer empty? */
    Status.Flags.Word = SPI1DR;  /* Read data from receiver */

    NOP();NOP();NOP();NOP();
    NOP();NOP();NOP();NOP();
    FR_CSN_B = 1;
    Status.Version.Word = 0xFFFF; 
  } else {
    Status.Flags.Word = 0xFFFF;
    Status.Version.Word = 0xFFFF; 
  }
 
  return Status;
}

/* Channel A+B Status Register readout */
void fr_bd_read_status_AB(Fr_bd_status_register *status1, Fr_bd_status_register *status2) {
  *status1 = fr_bd_read_status_A();
  *status2 = fr_bd_read_status_B();
}


/*  */
void fr_bd_change_mode(Fr_channel_type channel, Fr_BD_mode_type mode)               
{    
    switch(channel) {
    case FR_CHANNEL_A: 
      switch(mode) {
        case FR_BD_NORMAL : 
          FR_STBN_A = 1;
          break;      
        case FR_BD_STANDBY : 
          FR_STBN_A = 0;
          break;
        default: break;
      }break;
    case FR_CHANNEL_B: 
      switch(mode) {
        case FR_BD_NORMAL : 
          FR_STBN_B = 1;
          break;      
        case FR_BD_STANDBY : 
          FR_STBN_B = 0;
          break;
        default: break;
      }break;
    case FR_CHANNEL_AB: 
      switch(mode) {
        case FR_BD_NORMAL : 
          FR_STBN_A = 1;
          FR_STBN_B = 1;
          break;      
        case FR_BD_STANDBY : 
          FR_STBN_A = 0;
          FR_STBN_B = 0;
          break;
        default: break;
      }break;
    default: break;
  }
}

/* Change Error indication mode (Latched/tracked) */
void fr_bd_change_error_mode(Fr_channel_type channel, Fr_BD_error_mode_type mode) {
  switch(channel) {
    case FR_CHANNEL_A: 
      switch(mode) {
        case FR_BD_LATCHED : 
          FR_SCK_A = 0;
          FR_CSN_A = 1;
          fr_bd_error_mode_A = FR_BD_LATCHED; 
          break;      
        case FR_BD_TRACK : 
          FR_SCK_A = 1;
          FR_CSN_A = 0;
          fr_bd_error_mode_A = FR_BD_TRACK; 
          break;
        default: break;
      }break;
    case FR_CHANNEL_B: 
      switch(mode) {
        case FR_BD_LATCHED : 
          FR_SCK_B = 0;
          FR_CSN_B = 1;
          fr_bd_error_mode_B = FR_BD_LATCHED;
          break;      
        case FR_BD_TRACK : 
          FR_SCK_B = 1;
          FR_CSN_B = 0;
          fr_bd_error_mode_B = FR_BD_TRACK;
          break;
        default: break;
      }break;
    case FR_CHANNEL_AB: 
      switch(mode) {
        case FR_BD_LATCHED : 
          FR_SCK_A = 0;
          FR_SCK_B = 0;
          FR_CSN_A = 1;
          FR_CSN_B = 1;
          fr_bd_error_mode_A = FR_BD_LATCHED;
          fr_bd_error_mode_B = FR_BD_LATCHED;
          break;      
        case FR_BD_TRACK : 
          FR_SCK_A = 1;
          FR_SCK_B = 1;
          FR_CSN_A = 0;
          FR_CSN_B = 0;
          fr_bd_error_mode_A = FR_BD_TRACK;
          fr_bd_error_mode_B = FR_BD_TRACK;
          break;
        default: break;
      }break;
    default: break;
  }
}

/* Read current Error indication mode */
Fr_BD_error_mode_type fr_bd_read_error_indication_mode(Fr_channel_type channel) {
  Fr_BD_error_mode_type mode;  
  switch(channel) {
    case FR_CHANNEL_A: 
      mode = fr_bd_error_mode_A;
      break;
    case FR_CHANNEL_B: 
      mode = fr_bd_error_mode_B;
      break;
    case FR_CHANNEL_AB: 
      if (fr_bd_error_mode_A == fr_bd_error_mode_B) mode = fr_bd_error_mode_A;
      else mode = FR_BD_OTHER;
      break;
    default: break;
  }
  
  return mode;
}

/* Application Test Test Patterns */

/* Valid Remote Wakeup Test Patterns */
void TestPatternRemoteWakeup(Fr_channel_type channel) {
  byte i;
                            
  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100]
      // 18us
      NOP();               // [25]
      i=238; while(--i);   // [15875] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_A(); // [100]  
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
      // 18us
      NOP();               // [25]
      i=238; while(--i);   // [15875] 25ns*(1+3*i)
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100] 
      // 18us
      NOP();               // [25]
      i=238; while(--i);   // [15875] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_B(); // [100]  
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
      // 18us
      NOP();               // [25]
      i=238; while(--i);   // [15875] 25ns*(1+3*i)
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
      // 18us
      NOP();                // [25]
      i=238; while(--i);    // [15875] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_AB(); // [100]   
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
      // 18us
      NOP();                // [25]
      i=238; while(--i);    // [15875] 25ns*(1+3*i)
  }
}

/* Valid Remote Wakeup Test Patterns - Long */
void TestPatternRemoteWakeupLong(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100]
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_A(); // [100]  
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100] 
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_B(); // [100]  
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
      // 10us
      i=132; while(--i);   // [9925] 25ns*(1+3*i)
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 10us
      i=132; while(--i);    // [9925] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
      // 10us
      i=132; while(--i);    // [9925] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_AB(); // [100]   
      // 10us
      i=132; while(--i);    // [9925] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
      // 10us
      i=132; while(--i);    // [9925] 25ns*(1+3*i)
  }
}

/* Alternative Remote Wakeup Test Patterns */
void TestPatternRemoteWakeupAlternative(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXD_A = 1;        // [100]
      // 18us
      NOP();               // [25]
      i=238; while(--i);   // [15875] 25ns*(1+3*i)
      FR_TXD_A = 0;        // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXD_B = 1;        // [100]
      // 18us
      NOP();               // [25]
      i=238; while(--i);   // [15875] 25ns*(1+3*i)
      FR_TXD_B = 0;        // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXD_AB();      // [100]
      // 18us
      NOP();                // [25]
      i=238; while(--i);    // [15875] 25ns*(1+3*i)
      FR_CLR_TXD_AB();      // [100]   
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
  }
}


/* Shorted Remote Wakeup Test Patterns */
void TestPatternRemoteWakeupShorted(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100]
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_A(); // [100]  
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100] 
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_B(); // [100]  
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_CLR_TXD_TXEN_AB(); // [100]   
      // 4.1us
      i=53; while(--i);    // [4000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
  }
}

/* Remote Wakeup Frame Payload Test Patterns */
void TestPatternRemoteWakeupFrame(Fr_channel_type channel) {
  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A();  
                             
        asm LDAB  #5         // [25]
      asm START1A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START1A  // [75]
        
        asm LDAB  #5         // [25]
      asm START2A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START2A  // [75]
      
        asm LDAB  #5         // [25]
      asm START3A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START3A  // [75]
        
        asm LDAB  #5         // [25]
      asm START4A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START4A  // [75]
      
        asm LDAB  #5         // [25]
      asm START5A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START5A  // [75]
        
        asm LDAB  #5         // [25]
      asm START6A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START6A  // [75]    
      
        asm LDAB  #6         // [25]
      asm START7A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START7A  // [75]     

      FR_SET_TXD_TXEN_A(); //   [100] 
  } else if (channel == FR_CHANNEL_B) {
            //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_B();  
                             
        asm LDAB  #5         // [25]
      asm START1B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START1B  // [75]
        
        asm LDAB  #5         // [25]
      asm START2B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START2B  // [75]
      
        asm LDAB  #5         // [25]
      asm START3B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START3B  // [75]
        
        asm LDAB  #5         // [25]
      asm START4B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START4B  // [75]
      
        asm LDAB  #5         // [25]
      asm START5B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START5B  // [75]
        
        asm LDAB  #5         // [25]
      asm START6B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START6B  // [75]    
      
        asm LDAB  #6         // [25]
      asm START7B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START7B  // [75]     

      FR_SET_TXD_TXEN_B(); //   [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      
  }
}

/* Wrong Remote Wakeup Frame Payload Test Patterns */
void TestPatternRemoteWakeupFrameWrong(Fr_channel_type channel) {
  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A();  
                             
        asm LDAB  #5         // [25]
      asm START1A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START1A  // [75]
        
        asm LDAB  #5         // [25]
      asm START2A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START2A  // [75]
      
        asm LDAB  #5         // [25]
      asm START3A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START3A  // [75]
        
        asm LDAB  #5         // [25]
      asm START4A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START4A  // [75]
      
        asm LDAB  #5         // [25]
      asm START5A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START5A  // [75]
        
        asm LDAB  #5         // [25]
      asm START6A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START6A  // [75]    
      
        asm LDAB  #6         // [25]
      asm START7A:  
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 0;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 0;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100]
        FR_TXD_A = 1;        // [100] 
        FR_TXD_A = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START7A  // [75]     

      FR_SET_TXD_TXEN_A(); //   [100] 
  } else if (channel == FR_CHANNEL_B) {
            //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_B();  
                             
        asm LDAB  #5         // [25]
      asm START1B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START1B  // [75]
        
        asm LDAB  #5         // [25]
      asm START2B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START2B  // [75]
      
        asm LDAB  #5         // [25]
      asm START3B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START3B  // [75]
        
        asm LDAB  #5         // [25]
      asm START4B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START4B  // [75]
      
        asm LDAB  #5         // [25]
      asm START5B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START5B  // [75]
        
        asm LDAB  #5         // [25]
      asm START6B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 0;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 0;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START6B  // [75]    
      
        asm LDAB  #6         // [25]
      asm START7B:  
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 0;        // [100] 
        FR_TXD_B = 1;        // [100]
        FR_TXD_B = 1;        // [100] 
        FR_TXD_B = 1;        // [100] 
        asm NOP              // [25]
        asm DBNE  B,START7B  // [75]     

      FR_SET_TXD_TXEN_B(); //   [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      
  }
}



/* Non Valid Remote Wakeup Test Patterns - short IDLE*/
void TestPatternRemoteWakeupFailShortIdle(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_A = 1;       // [100]
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_TXEN_A = 0;       // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_B = 1;       // [100]
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_TXEN_B = 0;       // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXEN_AB();     // [100]
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_CLR_TXEN_AB();     // [100]   
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
  }
}

/* Non Valid Remote Wakeup Test Patterns - long IDLE */
void TestPatternRemoteWakeupFailLongIdle(Fr_channel_type channel) {
  byte i;
  word j;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_A = 1;       // [100]
      // 140us
      NOP(); NOP();        // [50]
      j=1864; while(--j);  // [139850] 25ns*(2+3*i)
      FR_TXEN_A = 0;       // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_B = 1;       // [100]
      // 140us
      NOP(); NOP();        // [50]
      j=1864; while(--j);  // [139850] 25ns*(2+3*i)
      FR_TXEN_B = 0;       // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100] 
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXEN_AB();     // [100]
      // 140us
      NOP(); NOP();        // [50]
      j=1864; while(--j);  // [139850] 25ns*(2+3*i)
      FR_CLR_TXEN_AB();     // [100]   
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
  }
}

/* Non Valid Remote Wakeup Test Patterns - short firs LOW */
void TestPatternRemoteWakeupFailShortLow1(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_TXEN_A = 1;       // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_A = 0;       // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_TXEN_B = 1;       // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_B = 0;       // [100] 
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_SET_TXEN_AB();     // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_CLR_TXEN_AB();     // [100]   
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
  }
}

/* Non Valid Remote Wakeup Test Patterns - short second LOW */
void TestPatternRemoteWakeupFailShortLow2(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_A = 1;       // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_A = 0;       // [100] 
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_B = 1;       // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_TXEN_B = 0;       // [100] 
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100]
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 6us
      NOP();                // [25]
      i=78; while(--i);     // [5875] 25ns*(1+3*i)
      FR_SET_TXEN_AB();     // [100]
      // 6us
      NOP();               // [25]
      i=78; while(--i);    // [5875] 25ns*(1+3*i)
      FR_CLR_TXEN_AB();     // [100]   
      // 0.9us
      NOP();               // [25]
      i=10; while(--i);    // [775] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100]
  }
}













 /* One 10gdBit High Test Pattern */
void TestPattern_HIGH(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_TXD_A = 1;        //   [100]
      FR_TXEN_A = 0;       //   [100]       
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_TXEN_A = 1;       //   [100]
  } else if (channel == FR_CHANNEL_B) {
      FR_TXD_B = 1;        //   [100]
      FR_TXEN_B = 0;       //   [100]       
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_TXEN_B = 1;       //   [100]
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_SET_TXD_AB();     //   [100]
      FR_CLR_TXEN_AB();    //   [100]       
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_SET_TXEN_AB();    //   [100]  
  }
}

 /* One 10gdBit LOW Test Pattern */
void TestPattern_LOW(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); //   [100]       
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50] +
      FR_SET_TXD_TXEN_A(); //   [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); //   [100]       
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50] +
      FR_SET_TXD_TXEN_B(); //   [100]
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB();//   [100]       
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50] +
      FR_SET_TXD_TXEN_AB();//   [100]
  }
}



/* TSS Test Patterns */
void TestPatternTSS(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]
      // 11 gdBit
      i=13; while(--i);    // [1000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_A(); // [100] 

  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]
      // 11 gdBit
      i=13; while(--i);    // [1000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_B(); // [100] 
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB(); // [100]
      // 11 gdBit
      i=13; while(--i);    // [1000] 25ns*(1+3*i)
      FR_SET_TXD_TXEN_AB(); // [100] 
  }
}


/* TSS + one 10gdBit High Test Pattern */
void TestPatternTSS_HIGH(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_TXD_A = 1;        //   [100] 
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_SET_TXD_TXEN_A(); //   [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_TXD_B = 1;        //   [100] 
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_SET_TXD_TXEN_B(); //   [100]
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB(); 
      // 11 gdBit TSS
      i=13; while(--i);     // [1000] 25ns*(1+3*i)
      FR_SET_TXD_AB();      // [100] 
      // 10 gdBit HIGH
      i=11; while(--i);     // [850] 25ns*(1+3*i)
      NOP(); NOP();         // [50]
      FR_SET_TXD_TXEN_AB(); // [100] 
  }
}

/* TSS + one 10gdBit LOW Test Pattern */
void TestPatternTSS_LOW(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); 
      // 11 gdBit TSS + 10 gdBit LOW // [2100]=
      i=26; while(--i);    //   [1975]+ 
      NOP();               //   [25]+
      FR_SET_TXD_TXEN_A(); //   [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); 
      // 11 gdBit TSS + 10 gdBit LOW // [2100]=
      i=26; while(--i);    //   [1975]+ 
      NOP();               //   [25]+
      FR_SET_TXD_TXEN_B(); //   [100] 
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB(); 
      // 11 gdBit TSS + 10 gdBit LOW // [2100]=
      i=26; while(--i);     //   [1975]+ 
      NOP();                //   [25]+
      FR_SET_TXD_TXEN_AB(); //   [100] 
  }
}

/* TSS + one 10gdBit High + one 10gdBit LOW Test Pattern */
void TestPatternTSS_HIGH_LOW(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_TXD_A = 1;        //   [100] 
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_TXD_A = 0;        //   [100] 
      // 10 gdBit LOW      // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_SET_TXD_TXEN_A(); //   [100]  
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_TXD_B = 1;        //   [100] 
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_TXD_B = 0;        //   [100] 
      // 10 gdBit LOW      // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_SET_TXD_TXEN_B(); //   [100]  
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_SET_TXD_AB();     //   [100] 
      // 10 gdBit HIGH     // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_CLR_TXD_AB();     //   [100] 
      // 10 gdBit LOW      // [1000]=
      i=11; while(--i);    //   [850]+ 
      NOP(); NOP();        //   [50]+
      FR_SET_TXD_TXEN_AB(); //  [100]  
  }
}

/* TSS + one 50/50 Test Pattern */
void TestPatternTSS_50_50(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_TXD_A = 1;        //   [100] 
      // One 50/50 test pattern
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_SET_TXD_TXEN_A(); //   [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_TXD_B = 1;        //   [100] 
      // One 50/50 test pattern
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_SET_TXD_TXEN_B(); //   [100] 
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB(); 
      // 11 gdBit TSS      // [1100]=
      i=13; while(--i);    //   [1000]+ 
      FR_SET_TXD_AB();     //   [100] 
      // One 50/50 test pattern
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100]
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_TXEN_AB(); //  [100]  
  }
}

/* TSS + one 10/90 Test Pattern */
void TestPatternTSS_10_90(Fr_channel_type channel) {
  byte i;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); 
      // 11 gdBit TSS      //   [1100]=
      i=13; while(--i);    //     [1000]+ 
      FR_TXD_A = 1;        //     [100] 
      // One 10/90 test pattern [1000]=
      FR_TXD_A = 0;        //     [100]+
      i=10; while(--i);    //     [775]+
      NOP();               //     [25]+
      FR_SET_TXD_TXEN_A(); //     [100] 
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); 
      // 11 gdBit TSS      //   [1100]=
      i=13; while(--i);    //     [1000]+ 
      FR_TXD_B = 1;        //     [100] 
      // One 10/90 test pattern [1000]=
      FR_TXD_B = 0;        //     [100]+
      i=10; while(--i);    //     [775]+
      NOP();               //     [25]+
      FR_SET_TXD_TXEN_B(); //     [100] 
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_CLR_TXD_TXEN_AB(); 
      // 11 gdBit TSS      //   [1100]=
      i=13; while(--i);    //     [1000]+ 
      FR_SET_TXD_AB();     //     [100] 
      // One 10/90 test pattern [1000]=
      FR_CLR_TXD_AB();     //     [100]+
      i=10; while(--i);    //     [775]+
      NOP();               //     [25]+
      FR_SET_TXD_TXEN_AB(); //    [100] 
  }
}

/* Two 50/50 Test Patterns */
void TestPattern_Two_50_50(Fr_channel_type channel) {
  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_TXD_A = 1;        //   [100]      
      FR_TXEN_A = 0;       //   [100]
      
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]
      FR_TXD_A = 1;        //   [100] 
      FR_TXD_A = 0;        //   [100]     
      FR_SET_TXD_TXEN_A(); //   [100]

  } else if (channel == FR_CHANNEL_B) {
      FR_TXD_B = 1;        //   [100]      
      FR_TXEN_B = 0;       //   [100]
      
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]
      FR_TXD_B = 1;        //   [100] 
      FR_TXD_B = 0;        //   [100]     
      FR_SET_TXD_TXEN_B(); //   [100]
  }else if (channel == FR_CHANNEL_AB)  { 
      FR_SET_TXD_AB();     //   [100]
      FR_CLR_TXEN_AB(); 
      
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100]
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100]
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_AB();     //   [100] 
      FR_CLR_TXD_AB();     //   [100]
      FR_SET_TXD_TXEN_AB(); //  [100]  
  }
}


/* Current Measurement Test Patterns - long IDLE */
void TestPatternCurrentMeasurement(Fr_channel_type channel) {
  byte i;
  word j;

  if (channel == FR_CHANNEL_A) {
      //                      [time in ns]
      //=========================================
      FR_CLR_TXD_TXEN_A(); // [100]    
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i)      
      FR_SET_TXD_TXEN_A(); // [100]
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i) 
      FR_CLR_TXD_TXEN_A(); // [100]     
      // 1us
      NOP();NOP();         // [50]
      i=11; while(--i);    // [850] 25ns*(1+3*i)     
      FR_TXD_A = 1;        // [100]
      // 649us
      NOP();               // [25]
      j=8651; while(--j);  // [648875] 25ns*(2+3*i) 
      FR_SET_TXD_TXEN_A(); // [100] 
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i)       
  } else if (channel == FR_CHANNEL_B) {
      FR_CLR_TXD_TXEN_B(); // [100]    
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i)      
      FR_SET_TXD_TXEN_B(); // [100]
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i) 
      FR_CLR_TXD_TXEN_B(); // [100]     
      // 1us
      NOP();NOP();         // [50]
      i=11; while(--i);    // [850] 25ns*(1+3*i)     
      FR_TXD_B = 1;        // [100]
      // 649us
      NOP();               // [25]
      j=8651; while(--j);  // [648875] 25ns*(2+3*i) 
      FR_SET_TXD_TXEN_B(); // [100] 
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i)  
  }else if (channel == FR_CHANNEL_AB)  {  
      FR_CLR_TXD_TXEN_AB(); // [100]    
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i)      
      FR_SET_TXD_TXEN_AB();// [100]
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i) 
      FR_CLR_TXD_TXEN_AB();// [100]     
      // 1us
      NOP();NOP();         // [50]
      i=11; while(--i);    // [850] 25ns*(1+3*i)     
      FR_SET_TXD_AB();     // [100]
      // 649us
      NOP();               // [25]
      j=8651; while(--j);  // [648875] 25ns*(2+3*i) 
      FR_SET_TXD_TXEN_AB();// [100] 
      // 650us
      NOP();NOP();         // [50]
      j=8664; while(--j);  // [649850] 25ns*(2+3*i)  
  }
}


/* Configuration of CPU pins connected to FR bus driver */
void Fr_bd_ports_init(void) {
  /* PORTx  Port x Data Register */;
  /* DDRx   Port x Data Direction Register (0-input / 1-output) */
  /*                        */
  /* UV_VIO----++----BGE_A  */
  /* UV_VCC---+||+---STBN_A */
  /* SD_VIO--+||||+--CSN_A  */
  /* SD_VCC-+||||||+-ERRN_A */
  /*        ||||||||        */
  /*        VVVVVVVV        */
  PORTA|= 0b00001100;   /* Port A Data */
  PORTA&= 0b11111110;   /* Port A Data */
  DDRA |= 0b00001100;   /* Port A (0-input / 1-output) */ 
  DDRA &= 0b11111110;   /* Port A (0-input / 1-output) */ 
    
  /* CSN_B ---------+     */
  /* SDO_B --------+|     */
  /*               ||     */
  /*         VVVVVVVV     */
  PORTB |= 0b00000000;   /* Port B Data */
  PORTB &= 0b11111111;   /* Port B Data */
  DDRB  |= 0b00000000;   /* Port B (0-input / 1-output) */ 
  DDRB  &= 0b11111111;   /* Port B (0-input / 1-output) */ 
      
  /* XIRQ ---------+     */
  /* IRQ ---------+|     */
  /* BGE_B ------+||     */
  /* STBN_B ----+|||     */
  /* SCK_B ----+||||     */
  /* MODA ----+|||||     */
  /* MODB ---+||||||     */
  /* XCLK --+|||||||     */
  /*        ||||||||     */
  /*        VVVVVVVV     */
  PORTE|= 0b00001100;   /* Port E Data */
  PORTE&= 0b11111111;   /* Port E Data */
  DDRE |= 0b00001100;   /* Port E (0-input / 1-output) */  
  DDRE &= 0b11111111;   /* Port E (0-input / 1-output) */  
  
  /* PTx Port x Data Register */ 
  /* DDRx Port x Data Direction Register (0-input / 1-output) */ 
  
  /* RXD_A --------+     */
  /* TXD_A -------+|     */
  /* TXEN_A -----+||     */
  /*       -----+|||     */
  /* RXD_B ----+||||     */
  /* TXD_B ---+|||||     */
  /* TXEN_B -+||||||     */
  /*       -+|||||||     */
  /*        ||||||||     */
  /*        VVVVVVVV     */
  PTH  =  0b01110111;  /* Port H */ 
  DDRH =  0b11101110;  /* Port H (0-input / 1-output)*/ 
  
  /* FR CC STB1..4 outputs */
  /*      ----------0     */
  /*      ---------1|     */
  /*      --------2||     */
  /* STB0 -------3|||     */
  /* STB1 ------4||||     */
  /* STB2 -----5|||||     */
  /* STB3 ----6||||||     */
  /* ERRN_B -7|||||||     */
  /*         ||||||||     */
  /*         VVVVVVVV     */ 
  PTJ  =   0b00000000;  /* Port J */ 
  DDRJ =   0b01111111;  /* Port J (0-input / 1-output) */ 
  
  /* CAN bus driver signals (CANRX as input) */
  
  /* RXCAN --------0     */
  /* TXCAN -------1|     */
  /* STBCAN -----2||     */
  /*       -----3|||     */
  /* SDO_A ----4||||     */
  /*       ---5|||||     */
  /* SCK_A --6||||||     */
  /*       -7|||||||     */
  /*        ||||||||     */
  /*        VVVVVVVV     */ 
  PTM |=  0b00000000;  /* Port M */
  PTM &=  0b11111111;  /* Port M */
  DDRM|=  0b00000000;  /* Port M (0-input / 1-output)*/ 
  DDRM&=  0b11111111;  /* Port M (0-input / 1-output)*/ 
  
  
}






/************************************************************************************/

